var userAgent = navigator.userAgent.toLowerCase();
var ua = {
    isAndroid: function () {
        return userAgent.includes('android');
    },
    isDingTalk: function () {
        return userAgent.includes('dingtalk');
    },
    isEdge: function () {
        return userAgent.includes('dingtalk');
    },
    isIE: function () {
        return (Boolean(window.ActiveXObject)) || 'ActiveXObject' in window;
    },
    isIOS: function () {
        return /ios|iphone|ipad|ipod/.test(userAgent);
    },
    isIpad: function () {
        return userAgent.includes('ipad');
    },
    isQQ: function () {
        return userAgent.includes('qq/');
    },
    isWeixin: function () {
        return /MicroMessenger/i.test(userAgent);
    },
    isMobile: function () {
        return /android|webos|iphone|ipad|ipod|blackberry|iemobile|opera mini/.test(userAgent);
    },
    isPC: function () {
        return !this.isMobile();
    }
};

var Cookie = {
    parse: function () {
        var cookie = document.cookie;
        return cookie
            .split(';')
            .map(function (item) { return item.split('='); })
            .reduce(function (result, item) {
            var _a = item[0], key = _a === void 0 ? '' : _a, _b = item[1], value = _b === void 0 ? '' : _b;
            result[decodeURIComponent(key.trim())] = decodeURIComponent(value.trim());
            return result;
        }, {});
    },
    /**
     * get cookie value via key
     * @param key
     */
    get: function (key) {
        if (key === null || key === undefined)
            return '';
        var cookieName = encodeURIComponent(key) + "=";
        var cookieStart = document.cookie.indexOf(cookieName);
        var result = '';
        if (cookieStart > -1) {
            var cookieEnd = document.cookie.indexOf(';', cookieStart);
            result = decodeURIComponent(document.cookie.substring(cookieStart + cookieName.length, cookieEnd !== -1 ? cookieEnd : document.cookie.length));
        }
        return result;
    },
    /**
     * set cookie
     * @param  {string} key
     * @param  {string} value
     * @param  {CookieOption} option
     */
    set: function (key, value, option) {
        var cookieString = encodeURIComponent(key) + "=" + encodeURIComponent(value);
        var expires = option.expires, path = option.path, domain = option.domain, secure = option.secure;
        if (expires !== null && expires !== undefined) {
            var expireString = expires instanceof Date ? expires.toUTCString() : expires;
            cookieString += "; expires=" + expireString;
        }
        if (path !== null && path !== undefined) {
            cookieString += "; path=" + path;
        }
        if (domain !== null && domain !== undefined) {
            cookieString += "; domain=" + domain;
        }
        if (secure !== null && secure !== undefined) {
            cookieString += '; secure';
        }
        document.cookie = cookieString;
    }
};

/**
 * open page by tag link
 * @param url
 * @param attributes
 * @returns
 */
var openByTagLink = function (url, attributes) {
    if (!url)
        return;
    var element = document.createElement('a');
    element.href = url;
    element.target = '_blank';
    typeof attributes === 'object' && Object.keys(attributes).forEach(function (attrKey) {
        element[attrKey] = attributes[attrKey];
    });
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
};

/**
 * string 返回type
 * @param {Any} target
 */
var getTypeOf = function (target) { return Object.prototype.toString
    .call(target)
    .slice(8, -1)
    .toLowerCase(); };

/**
 * safe json parse
 * @param args
 * @returns {[Error | null, any]}
 */
var JSONParse = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var value = null;
    var error = null;
    try {
        value = JSON.parse.apply(null, args);
    }
    catch (err) {
        error = err;
    }
    return [error, value];
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var CACHE_MAP = new Map();
var DEFAULT_EXPIRE = Number.MAX_SAFE_INTEGER;
/**
 * async cache
 * @param fn The function needs to be cached, return a promise
 * @param cacheOptions cache options
 * @param cacheOptions.key cache key
 * @param cacheOptions.expire ms number
 * @returns {Function} cached fn
 */
var cacheAsync = function (fn, cacheOptions) {
    if (cacheOptions === void 0) { cacheOptions = {}; }
    var key = cacheOptions.key, _a = cacheOptions.expire, expire = _a === void 0 ? DEFAULT_EXPIRE : _a;
    return function cacheAsyncAction() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        var context = this;
        var cacheKey = typeof key === 'function'
            ? key.apply(void 0, args) : key === null || key === undefined
            ? fn
            : key;
        var cache = CACHE_MAP.get(cacheKey);
        /**
         * set cache with fn invoke
         * @returns Promise<K>
         */
        var setCache = function () {
            var result = fn.apply(context, args);
            result = result instanceof Promise
                ? result
                : Promise.resolve(result);
            // set pending promise
            CACHE_MAP.set(cacheKey, {
                resolved: false,
                expire: +new Date() + expire,
                value: result
            });
            // record the fulfilled promise result
            result
                .then(function (res) {
                var firstTimeCache = CACHE_MAP.get(cacheKey);
                CACHE_MAP.set(cacheKey, __assign(__assign({}, firstTimeCache), { expire: +new Date() + expire, resolved: true, value: res }));
                return res;
            })
                .catch(function (error) {
                // remove the cache while the promise reject
                CACHE_MAP.delete(cacheKey);
                return Promise.reject(error);
            });
            return result;
        };
        /**
         * first time to invoke fn
         * set new cache info
         */
        if (!cache)
            return setCache();
        return cache.expire > +new Date()
            ? cache.resolved
                // hit the cache
                ? Promise.resolve(cache.value)
                // hit the pending promise
                : cache.value
            // cache expired
            : setCache();
    };
};
cacheAsync.CACHE_MAP = CACHE_MAP;

/**
 * is value can be parsed to a valid Date
 * @param target
 * @returns boolean
 */
var isDateValid = function (target) {
    if (target === undefined || target === null)
        return false;
    var toDate = target instanceof Date
        ? target
        : new Date(target);
    return !Number.isNaN(toDate.getTime());
};

/**
 * is plain object or not
 * @param target
 * @returns boolean
 */
var isPlainObject = function (target) { return Object.prototype
    .toString
    .call(target) === '[object, Object]'; };

var REG_EMAIL = /[\w!#$%&'*+/=?^_`{|}~-]+(?:\.[\w!#$%&'*+/=?^_`{|}~-]+)*@(?:[\w](?:[\w-]*[\w])?\.)+[\w](?:[\w-]*[\w])?/;
/**
 * 校验邮箱地址
 * @param {String} target
 */
var isEmail = function (target) { return REG_EMAIL.test(target); };

/**
 * 检测是否为数值
 * @param {Any} target
 */
var isNumber = function (target) { return /^\d+(\.\d+)?$/.test(target); };

var isTel = function (val) { return /^([0-9]{3,4}-)?[0-9]{7,8}$/.test(val); };
var isMob = function (val) { return /^((\+(86)|(86))\s)?(1)\d{10}/.test(val); };
/**
 * 校验电话号码
 * @param {String} target
 */
var isPhone = function (target) { return isTel(target) || isMob(target); };

/**
 * 判断是否图片
 * @param {String} target
 */
var isImage = function (target) { return /\.(jpeg|jpg|png|bmp|gif|wbmp|svg)(\?[\s\S]+)?$/i.test(target); };

/**
 * promisify setTimeout
 * @param ms micro second
 * @returns Promise
 */
var delay = function (ms) { return new Promise(function (resolve) {
    setTimeout(resolve, ms);
}); };

// eslint-disable-next-line @typescript-eslint/no-unused-vars
var noop = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
};

export { Cookie, JSONParse, cacheAsync, delay, getTypeOf, isDateValid, isEmail, isImage, isNumber, isPhone, isPlainObject, noop, openByTagLink, ua };
